import {
    NativeModules,
    DeviceEventEmitter,
} from 'react-native';

const RNUnityAds = NativeModules.RNUnityAds;

const eventHandlers = {
    onReady: new Map(),
    onStart: new Map(),
    onFinish: new Map(),
    onError: new Map(),
};

const addEventListener = (type, handler) => {
    switch (type) {
        case 'onReady':
            eventHandlers[type].set(handler, DeviceEventEmitter.addListener(type, handler));
            break;
        case 'onError':
            eventHandlers[type].set(handler, DeviceEventEmitter.addListener(type, (error) => { handler(error); }));
            break;
        case 'onStart':
            eventHandlers[type].set(handler, DeviceEventEmitter.addListener(type, handler));
            break;
        case 'onFinish':
            eventHandlers[type].set(handler, DeviceEventEmitter.addListener(type, handler));
            break;
        default:
            console.log(`Event with type ${type} does not exist.`);
    }
}

const removeEventListener = (type, handler) => {
    if (!eventHandlers[type].has(handler)) {
        return;
    }
    eventHandlers[type].get(handler).remove();
    eventHandlers[type].delete(handler);
}

const removeAllListeners = () => {
    DeviceEventEmitter.removeAllListeners('onReady');
    DeviceEventEmitter.removeAllListeners('onError');
    DeviceEventEmitter.removeAllListeners('onStart');
    DeviceEventEmitter.removeAllListeners('onFinish');
};

module.exports = {
    ...RNUnityAds,
    addEventListener,
    removeEventListener,
    removeAllListeners
};
